package vn.teko.loyalty.loyalty_flutter

import vn.teko.loyalty.core.model.exception.LoyaltyException

internal object ResultCode {
    const val success = "Success"
    private const val cannotCallLoyaltyService = "CannotCallLoyaltyService"
    private const val dataInactive = "DataInactive"
    private const val duplicateMemberCardId = "DuplicateMemberCardId"
    private const val duplicatePhoneNumber = "DuplicatePhoneNumber"
    private const val invalidParam = "InvalidParam"
    private const val invalidParamFormat = "InvalidParamFormat"
    private const val memberInactive = "MemberInactive"
    private const val memberQrInactive = "MemberQrInactive"
    private const val missingParam = "MissingParam"
    private const val paramOutOfRange = "ParamOutOfRange"
    private const val permissionDenied = "PermissionDenied"
    private const val redemptionCodeExpired = "RedemptionCodeExpired"
    private const val subjectNotFound = "SubjectNotFound"
    private const val systemError = "SystemError"
    private const val unauthorized = "Unauthorized"
    private const val unexpected = "Unexpected"
    private const val merchantNotMatch = "MemberNotMatchMerchant"
    private const val networkInactive = "NetworkInactive"

    fun mapLoyaltyExceptionToResultCode(exception: Throwable): String {
        return when (exception) {
            is LoyaltyException.InvalidParamFormatException -> invalidParamFormat
            is LoyaltyException.ParamOutOfRangeException -> paramOutOfRange
            is LoyaltyException.MissingParamException -> missingParam
            is LoyaltyException.InvalidParamException -> invalidParam
            is LoyaltyException.UnauthorizedException -> unauthorized
            is LoyaltyException.PermissionDenied -> permissionDenied
            is LoyaltyException.SubjectNotFoundException -> subjectNotFound
            is LoyaltyException.MemberNotMatchMerchantException -> merchantNotMatch
            is LoyaltyException.RedemptionCodeExpired -> redemptionCodeExpired
            is LoyaltyException.SubjectInactiveException.MemberInactiveException -> memberInactive
            is LoyaltyException.SubjectInactiveException.MemberQrCodeInactiveException -> memberQrInactive
            is LoyaltyException.SubjectInactiveException.DataInactiveException -> dataInactive
            is LoyaltyException.SubjectInactiveException.NetworkInactiveException -> networkInactive
            is LoyaltyException.ServerErrorException -> systemError
            is LoyaltyException.CannotCallLoyaltyService -> cannotCallLoyaltyService
            is LoyaltyException.DuplicateMemberCardId -> duplicateMemberCardId
            is LoyaltyException.DuplicatePhoneNumber -> duplicatePhoneNumber
            else -> unexpected
        }
    }
}