import 'package:auth_flutter/provider/username_password_provider.dart';
import 'package:auth_flutter/terra_auth.dart';
import 'package:flutter/material.dart';

import 'main.dart';

class LoginScreen extends StatefulWidget {
  const LoginScreen({Key? key}) : super(key: key);

  @override
  State<LoginScreen> createState() => _LoginScreenState();
}

class _LoginScreenState extends State<LoginScreen> {
  final _usernameController = TextEditingController();
  final _passwordController = TextEditingController();
  late TerraAuth _terraAuth;

  @override
  void initState() {
    super.initState();
    _usernameController.text = "trung.cs@teko.vn";
    _passwordController.text = "trung123456";

    _initTerraLibs();
  }

  void _initTerraLibs() async {
    try {
      _terraAuth = await TerraAuth.getInstance(terraAppName);
      print("Initialized Terra succeeded");
    } catch (error) {
      print("Initialized Terra failed: $error");
    }

  @override
  void dispose() {
    _usernameController.dispose();
    _passwordController.dispose();

    super.dispose();
  }

    void showToast(String message, BuildContext context) {
      final snackBar = SnackBar(
        content:  Text(message),
      );
      ScaffoldMessenger.of(context).showSnackBar(snackBar);
    }

}

  @override
  Widget build(BuildContext context) {
   return Scaffold(
     body: Padding(
       padding: const EdgeInsets.all(8.0),
       child: Column(
         children: [
           const SizedBox(height: 32,),
           Row(
             mainAxisAlignment: MainAxisAlignment.spaceEvenly,
             children: [
               Flexible(
                   flex: 1,
                   child: TextFormField(
                     controller: _usernameController,
                     decoration: const InputDecoration(
                       contentPadding: EdgeInsets.symmetric(vertical: 0, horizontal: 12),
                       hintText: "Username",
                       border: OutlineInputBorder(
                           borderRadius: BorderRadius.all(Radius.circular(5.0)),
                           borderSide: BorderSide(color: Colors.blue)),
                     ),
                   )),
               const SizedBox(
                 width: 12,
               ),
               Flexible(
                 flex: 1,
                 child: TextFormField(
                   controller: _passwordController,
                   decoration: const InputDecoration(
                     contentPadding: EdgeInsets.symmetric(vertical: 0, horizontal: 12),
                     hintText: "Password",
                     border: OutlineInputBorder(
                         borderRadius: BorderRadius.all(Radius.circular(5.0)),
                         borderSide: BorderSide(color: Colors.blue)),
                   ),
                 ),
               ),
             ],
           ),
           Builder(
               builder: (context) {
                 return ElevatedButton(
                     onPressed: () async {
                       final credential = UsernamePasswordProvider.createCredential(
                           _usernameController.text.trim(), _passwordController.text.trim());
                       final result = await _terraAuth.loginWithCredential(credential);
                       showToast("Login Result: ${result.isSuccess}", context);
                     },
                     child: const Text("Username Login"));
               }
           ),
           Builder(
               builder: (context) {
                 return Row(
                   mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                   children: [
                     ElevatedButton(
                         onPressed: () async {
                           final result = await _terraAuth.loginWithGoogle();
                           showToast("Login Result: ${result.isSuccess}", context);
                         },
                         child: const Text("Google Login")),
                     ElevatedButton(
                         onPressed: () async {
                           final result = await _terraAuth.loginWithFacebook();
                           showToast("Login Result: ${result.isSuccess}", context);
                         },
                         child: const Text("Facebook Login")),
                   ],
                 );
               }
           ),
           ElevatedButton(
               onPressed: () async {
                 final result = await _terraAuth.logout();
                 showToast("Logout: ${result.isSuccess}", context);
               },
               child: const Text("Logout")),
           ElevatedButton(
               onPressed: () async {
                 Navigator.of(context).pop();
               },
               child: const Text("Back")),
         ],
       ),
     ),
   );
  }

  void showToast(String s, BuildContext context) {
    final snackBar = SnackBar(
      content:  Text(s)
    );

    // Find the ScaffoldMessenger in the widget tree
    // and use it to show a SnackBar.
    ScaffoldMessenger.of(context).showSnackBar(snackBar);
  }
}

