//
//  ResultCode.swift
//  loyalty_core_flutter
//
//  Created by Cao Trung on 15/07/2022.
//

import Foundation
import LoyaltyModel

struct LoyaltyExceptionData {
    let code: String
    let message: String?
}

let success = "Success"
let cannotCallLoyaltyService = "CannotCallLoyaltyService"
let dataInactive = "DataInactive"
let duplicateMemberCardId = "DuplicateMemberCardId"
let duplicatePhoneNumber = "DuplicatePhoneNumber"
let invalidParam = "InvalidParam"
let invalidParamFormat = "InvalidParamFormat"
let memberInactive = "MemberInactive"
let memberQrInactive = "MemberQrInactive"
let missingParam = "MissingParam"
let paramOutOfRange = "ParamOutOfRange"
let permissionDenied = "PermissionDenied"
let redemptionCodeExpired = "RedemptionCodeExpired"
let subjectNotFound = "SubjectNotFound"
let systemError = "SystemError"
let unauthorized = "Unauthorized"
let unexpected = "Unexpected"
let merchantNotMatch = "MemberNotMatchMerchant"
let networkInactive = "NetworkInactive"

func convertCodeToException(loyaltyError: LoyaltyError) -> LoyaltyExceptionData {
    switch loyaltyError {
    case .cannotCallLoyaltyService:
        return LoyaltyExceptionData(code: cannotCallLoyaltyService, message: loyaltyError.message)
    case .dataInactive:
        return LoyaltyExceptionData(code: dataInactive, message: loyaltyError.message)
    case .duplicateMemberCardId:
        return LoyaltyExceptionData(code: duplicateMemberCardId, message: loyaltyError.message)
    case .duplicatePhone:
        return LoyaltyExceptionData(code: duplicatePhoneNumber, message: loyaltyError.message)
    case .invalidParam(let message):
        return LoyaltyExceptionData(code: invalidParam, message: message)
    case .invalidParamFormat(let message):
        return LoyaltyExceptionData(code: invalidParamFormat, message: message)
    case .memberInactive:
        return LoyaltyExceptionData(code: memberInactive, message: loyaltyError.message)
    case .memberQrInactive:
        return LoyaltyExceptionData(code: memberQrInactive, message: loyaltyError.message)
    case .missingParam(let message):
        return LoyaltyExceptionData(code: missingParam, message: message)
    case .paramOutOfRange(let message):
        return LoyaltyExceptionData(code: paramOutOfRange, message: message)
    case .permissionDenied:
        return LoyaltyExceptionData(code: permissionDenied, message: loyaltyError.message)
    case .redemptionCodeExpired:
        return LoyaltyExceptionData(code: redemptionCodeExpired, message: loyaltyError.message)
    case .subjectNotFound(let message):
        return LoyaltyExceptionData(code: subjectNotFound, message: message)
    case .unexpected:
        return LoyaltyExceptionData(code: systemError, message: loyaltyError.message)
    case .unauthorized:
        return LoyaltyExceptionData(code: unauthorized, message: loyaltyError.message)
    case .wrongMerchant:
        return LoyaltyExceptionData(code: merchantNotMatch, message: loyaltyError.message)
    case .networkInactive:
        return LoyaltyExceptionData(code: networkInactive, message: loyaltyError.message)
    default:
        return LoyaltyExceptionData(code: unexpected, message: loyaltyError.message)
    }
}
