import '../model/loyalty_configuration.dart';
import '../model/member_info.dart';
import '../model/membership_qr.dart';
import '../model/netword_config_result.dart';
import '../model/register_member_request.dart';
import '../model/status.dart';
import '../model/transaction_history.dart';
import '../model/transaction_history_request.dart';
import '../model/transaction_type.dart';
import 'loyalty_api.dart';

class ModelConverter {
  ModelConverter._();

  static LoyaltyConfiguration toModelLoyaltyConfiguration(
      FlutterLoyaltyConfiguration flutterLoyaltyConfiguration) {
    return LoyaltyConfiguration(
        url: flutterLoyaltyConfiguration.url ?? "",
        bffUrl: flutterLoyaltyConfiguration.bffUrl ?? "",
        merchantCode: flutterLoyaltyConfiguration.merchantCode ?? "",
        clientId: flutterLoyaltyConfiguration.clientId ?? "",
        iamAudience: flutterLoyaltyConfiguration.iamAudience ?? "",
        linkMembershipUrl: flutterLoyaltyConfiguration.linkMembershipUrl ?? "",
        registerMemberUrl: flutterLoyaltyConfiguration.registerMemberUrl ?? "",
        allowLinkMembership: flutterLoyaltyConfiguration.allowLinkMembership ?? false,
        allowRegisterMembership: flutterLoyaltyConfiguration.allowRegisterMembership ?? false,
        membershipCodeFormat: flutterLoyaltyConfiguration.membershipCodeFormat ?? "");
  }

  static MemberInfo toModelMemberInfo(FlutterMemberInfo flutterMemberInfo) {
    return MemberInfo(
      memberQrCode: flutterMemberInfo.memberQrCode,
      memberId: flutterMemberInfo.memberId ?? "",
      phone: flutterMemberInfo.phone ?? "",
      memberCardId: flutterMemberInfo.memberCardId,
      name: flutterMemberInfo.name ?? "",
      isActive: flutterMemberInfo.isActive ?? false,
      point: flutterMemberInfo.point ?? 0,
      tierPoint: flutterMemberInfo.tierPoint,
      keepTierUntil: flutterMemberInfo.keepTierUntil,
      accumulationFrom: flutterMemberInfo.accumulationFrom,
      accumulationTo: flutterMemberInfo.accumulationTo,
      currentTierCode: flutterMemberInfo.currentTierCode,
      currentTierName: flutterMemberInfo.currentTierName,
      currentTierMinPoint: flutterMemberInfo.currentTierMinPoint,
      nextTierName: flutterMemberInfo.nextTierName,
      nextTierMinPoint: flutterMemberInfo.nextTierMinPoint,
      defaultCardType: flutterMemberInfo.defaultCardType.toCardType(),
    );
  }

  static EncryptedMemCode toModelEncryptedMemCode(FlutterMembershipQr membershipQr) {
    return EncryptedMemCode(
      qrData: membershipQr.qrData ?? "",
      issueAt: membershipQr.issueAt ?? 0,
      expireAt: membershipQr.expireAt ?? 0,
    );
  }

  static NetworkConfig toModelNetworkConfigResult(FlutterNetworkConfig networkData) {
    final networkTiers = networkData.networkTiers
        ?.map((e) => Tier(
              code: e?.code ?? "",
              name: e?.name ?? "",
              fromPoint: e?.fromPoint ?? 1,
              cardTieredImg: e?.cardTieredImg,
              tierIconImg: e?.tierIconImg,
            ))
        .toList();

    return NetworkConfig(
      networkId: networkData.networkId ?? "",
      networkName: networkData.networkName ?? "",
      exchangeRate: networkData.exchangeRate ?? 1,
      currencyImg: networkData.currencyImg ?? "",
      currencyName: networkData.currencyName ?? "",
      cardDefaultImg: networkData.cardDefaultImg,
      avatarDefaultImg: networkData.avatarDefaultImg,
      allowToGetMemberByPhone: networkData.allowToGetMemberByPhone ?? false,
      faqWebUrl: networkData.faqWebUrl,
      benefitWebUrl: networkData.benefitWebUrl,
      brandColor: networkData.brandColor,
      allowMerchantIssueMemberCard: networkData.allowMerchantIssueMemberCard ?? false,
      networkTiers: networkTiers,
    );
  }

  static TransactionHistory toModelTransactionHistory(FlutterTransactionHistory result) {
    final pagination = Pagination(
      page: result.pagination?.page ?? 0,
      pageSize: result.pagination?.pageSize ?? 0,
      totalItems: result.pagination?.totalItems ?? 0,
    );

    return TransactionHistory(
      transactions: result.transactions.toTxns() ?? [],
      pagination: pagination,
    );
  }

  static FlutterRegisterMemberRequest toFlutterRegisterMemberRequest(
      RegisterMemberRequest request) {
    return FlutterRegisterMemberRequest(
      phone: request.phone,
      name: request.name,
      address: request.address,
      dateOfBirth: request.dateOfBirth,
      email: request.email,
      gender: request.gender.toFlutterGender(),
      idCardNumber: request.idCardNumber,
      idCitizenNumber: request.idCitizenNumber,
      passportNumber: request.passportNumber,
    );
  }

  static FlutterTransactionHistoryRequest toFlutterTransactionHistoryRequest(
      TransactionHistoryRequest request) {
    return FlutterTransactionHistoryRequest(
      fromTime: request.fromTime,
      toTime: request.toTime,
      txnType: request.txnType.toFlutterTransactionRequestType(),
      page: request.page,
      pageSize: request.pageSize,
    );
  }
}

extension FlutterCardTypeX on FlutterCardType? {
  CardType? toCardType() {
    if (this != null) {
      if (this == FlutterCardType.memberCard) {
        return CardType.memberCard;
      } else {
        return CardType.qr;
      }
    }

    return null;
  }
}

extension ListRequestTransactionTypeX on RequestTransactionType? {
  FlutterTransactionRequestType? toFlutterTransactionRequestType() {
    switch (this) {
      case RequestTransactionType.grant:
        return FlutterTransactionRequestType.grant;
      case RequestTransactionType.spend:
        return FlutterTransactionRequestType.spend;
      case RequestTransactionType.all:
        return FlutterTransactionRequestType.all;
      default:
        return null;
    }
  }
}

extension GenderX on Gender? {
  FlutterGender toFlutterGender() {
    switch (this) {
      case Gender.male:
        return FlutterGender.male;
      case Gender.female:
        return FlutterGender.female;
      default:
        return FlutterGender.none;
    }
  }
}

extension ListFlutterTransactionsX on List<FlutterTransaction?>? {
  List<Transaction>? toTxns() {
    final transactions = this?.map((e) {
      TransactionType txnType;
      switch (e?.txnType) {
        case FlutterTransactionType.grant:
          txnType = TransactionType.grant;
          break;
        case FlutterTransactionType.spend:
          txnType = TransactionType.spend;
          break;
        case FlutterTransactionType.returnGrant:
          txnType = TransactionType.returnGrant;
          break;
        case FlutterTransactionType.returnSpend:
          txnType = TransactionType.returnSpend;
          break;
        default:
          txnType = TransactionType.grant;
          break;
      }

      AccountingType accountingType;
      switch (e?.accountingType) {
        case FlutterAccountingType.debit:
          accountingType = AccountingType.debit;
          break;
        default:
          accountingType = AccountingType.credit;
          break;
      }

      Status status;
      switch (e?.status) {
        case FlutterStatus.initFailure:
          status = Status.initFailure;
          break;
        case FlutterStatus.noAccounting:
          status = Status.noAccounting;
          break;
        case FlutterStatus.accountingSuccess:
          status = Status.accountingSuccess;
          break;
        case FlutterStatus.accountingFailure:
          status = Status.accountingFailure;
          break;
        case FlutterStatus.accounting:
          status = Status.accounting;
          break;
        default:
          status = Status.accountingSuccess;
          break;
      }

      return Transaction(
        id: e?.id ?? "",
        refId: e?.refId,
        orderCode: e?.orderCode,
        txnType: txnType,
        txnTypeText: e?.txnTypeText ?? "",
        txnDesc: e?.txnDesc,
        accountingType: accountingType,
        status: status,
        point: e?.point ?? 0,
        createdAt: e?.createdAt ?? 0,
        networkId: e?.networkId ?? 0,
        networkName: e?.networkName ?? "",
        merchantCode: e?.merchantCode,
        merchantName: e?.merchantName,
      );
    });

    return transactions?.toList();
  }
}