import '../exception/loyalty_core_exception.dart';
import 'loyalty_api.dart';

const _success = "Success";
const _cannotCallLoyaltyService = "CannotCallLoyaltyService";
const _dataInactive = "DataInactive";
const _duplicateMemberCardId = "DuplicateMemberCardId";
const _duplicatePhoneNumber = "DuplicatePhoneNumber";
const _invalidParam = "InvalidParam";
const _invalidParamFormat = "InvalidParamFormat";
const _memberInactive = "MemberInactive";
const _memberQrInactive = "MemberQrInactive";
const _missingParam = "MissingParam";
const _paramOutOfRange = "ParamOutOfRange";
const _permissionDenied = "PermissionDenied";
const _redemptionCodeExpired = "RedemptionCodeExpired";
const _subjectNotFound = "SubjectNotFound";
const _systemError = "SystemError";
const _unauthorized = "Unauthorized";
const _unexpected = "Unexpected";
const _merchantNotMatch = "MemberNotMatchMerchant";
const _networkInactive = "NetworkInactive";

Exception _convertedException({String? code, String? message}) {
  if (code == null) return Exception(message);

  switch (code) {
    case _cannotCallLoyaltyService:
      return CannotCallLoyaltyServiceException(message);
    case _dataInactive:
      return DataInactiveException(message);
    case _duplicateMemberCardId:
      return DuplicateMemberCardIdException(message);
    case _duplicatePhoneNumber:
      return DuplicatePhoneNumberException(message);
    case _invalidParam:
      return InvalidParamException(message);
    case _invalidParamFormat:
      return InvalidParamFormatException(message);
    case _memberInactive:
      return MemberInactiveException(message);
    case _memberQrInactive:
      return MemberQrCodeInactiveException(message);
    case _missingParam:
      return MissingParamException(message);
    case _paramOutOfRange:
      return ParamOutOfRangeException(message);
    case _permissionDenied:
      return PermissionDenied(message);
    case _redemptionCodeExpired:
      return RedemptionCodeExpired(message);
    case _subjectNotFound:
      return SubjectNotFoundException(message);
    case _systemError:
      return ServerErrorException(message);
    case _unauthorized:
      return UnauthorizedException(message);
    case _merchantNotMatch:
      return MemberNotMatchException(message);
    case _networkInactive:
      return NetworkInactiveException(message);
    default:
      return Exception(message);
  }
}

extension CommonResultX on CommonResult {
  bool get isFailed => code != _success;

  Exception getConvertedException() => _convertedException(code: code, message: error);
}

extension FlutterLoyaltyConfigurationResultX on FlutterLoyaltyConfigurationResult {
  bool get isFailed => code != _success;

  Exception getConvertedException() => _convertedException(code: code, message: error);
}

extension FlutterRegisterMemberResultX on FlutterRegisterMemberResult {
  bool get isFailed => code != _success;

  Exception getConvertedException() => _convertedException(code: code, message: error);
}

extension FlutterMembershipQrResultX on FlutterMembershipQrResult {
  bool get isFailed => code != _success;

  Exception getConvertedException() => _convertedException(code: code, message: error);
}

extension FlutterMemberInfoResultX on FlutterMemberInfoResult {
  bool get isFailed => code != _success;

  Exception getConvertedException() => _convertedException(code: code, message: error);
}

extension FlutterTransactionHistoryResultX on FlutterTransactionHistoryResult {
  bool get isFailed => code != _success;

  Exception getConvertedException() => _convertedException(code: code, message: error);
}

extension FlutterNetworkConfigResultX on FlutterNetworkConfigResult {
  bool get isFailed => code != _success;

  Exception getConvertedException() => _convertedException(code: code, message: error);
}

extension FlutterClientIdResultX on FlutterClientIdResult {
  bool get isFailed => code != _success;

  Exception getConvertedException() => _convertedException(code: code, message: error);
}
